#ifndef EM102_DYNAMIC_ARRAY_H
#define EM102_DYNAMIC_ARRAY_H 1

/*
  Define the minimum capacity for the dynamic array;
  the value of MIN_CAPACITY may be changed to anything in between 10
  and 100
*/
#define MIN_CAPACITY 10


/* The dynamic array data structure */

typedef struct {
  unsigned int capacity; /* the capacity of the dynamic array */
  unsigned int size;     /* the size of the dynamic array */
  double *a;             /* array containing the elements of the
			    dynamic array */
} DynamicArray;


/*
  Function: NewDynamicArray
  Usage: DynamicArray *da = NewDynamicArray();
  --------------------------------------------
  Returns a pointer to a newly created dynamic array, or NULL in case
  dynamic memory allocation fails.
  The new array has capacity MIN_CAPACITY and zero size.
*/
DynamicArray* NewDynamicArray(void);


/*
  Function: CopyDynamicArray
  Usage: da2 = CopyDynamicArray(da1);
  ------------------------------------
  Returns a copy of the dynamic array *da1, created using dynamic
  memory allocation. All data of *da1 are copied to the newly created
  array. If dynamic memory allocation fails the function returns NULL.
*/
DynamicArray* CopyDynamicArray(DynamicArray*);


/*
  Function: InsertInDynamicArray
  Usage: if ( InsertInDynamicArray(da, d) != NULL ) 
  -------------------------------------------------
  Inserts the number d in the dynamic array *da. If (before the
  insertion) the size of *da equals its capacity, its capacity is
  doubled and all previously inserted elements copied. The function
  returns NULL if dynamic memory allocation is used and failed,
  whereas a non-NULL pointer is returned otherwise.
*/
void* InsertInDynamicArray(DynamicArray*, double);


/*
  Function: DeleteFromDynamicArray
  Usage: if ( DeleteFromDynamicArray(da, pos) != NULL )
  -----------------------------------------------------
  Deletes from the dynamic array *da the element at position pos. If
  pos is greater than the size of the dynamic array minus 1, NULL is
  returned. If after the deletion the size of the dynamic array is
  less than or equal to half its capacity, then the capacity of the
  dynamic array is halved and all elements previously inserted in the
  array are properly copied. The only exception to the halfing rule
  is the capacity of the dynamic array must never be less than
  MIN_CAPACITY. If dynamic memory allocation fails (during the
  reduction of the dynamic array's capacity) NULL is returned,
  otherwise a non-NULL pointer is returned.
*/
void* DeleteFromDynamicArray(DynamicArray*, unsigned int);


/*
  Function: DynamicArraySize
  Usage: size = DynamicArraySize(da);
  -----------------------------------
  Returns the current size of the dynamic array.
*/
unsigned int DynamicArraySize(DynamicArray*);


/*
  Function: DynamicArrayCapacity
  Usage: c = DynamicArrayCapacity(da);
  ------------------------------------
  Returns the current capacity of the dynamic array.
*/
unsigned int DynamicArrayCapacity(DynamicArray*);


/*
  Function: PositionInDynamicArray
  Usage: pos = PositionInDynamicArray(da, d);
  -------------------------------------------
  Returns the left-most position of the element d within the dynamic
  array. If d is not one of the elements of *da, then
  DynamicArraySize(da)+1 is returned.
*/
unsigned int PositionInDynamicArray(DynamicArray*, double);


/*
  Function: ElementInDynamicArray
  Usage: dp = ElementInDynamicArray(da, k);
  -----------------------------------------
  Returns a pointer to element of the dynamic array at position k. If
  k is greater than the size of the dynamic array minus 1, NULL is
  returned.
*/
double* ElementInDynamicArray(DynamicArray*, unsigned int);


/*
  Function: FreeDynamicArray
  Usage: FreeDynamicArray(da);
  ----------------------------
  Frees the memory allocated for the dynamic array *da.
*/
void FreeDynamicArray(DynamicArray*);


#endif /* EM102_DYNAMIC_ARRAY_H */
