/*
  File: testDynamicArray.c
  ------------------------
  A simple test program has been implemented that tests the dynamic
  array data structure. The program reads the elements of the dynamic
  array from a file, inserts them in the dynamic array and then all
  elements are deleted one-by-one until the dynamic array is
  empty. Each element to be deleted is chosen at random among the
  existing elements in the dynamic array.
*/

#include <stdio.h>
#include <stdlib.h>
#include "DynamicArray.h"


/*
  Function: PrintDynamicArray
  Usage: PrintDynamicArray(da);
  -----------------------------
  Outputs to the screen the capacity, size and elements of the dynamic
  array *da.
*/
static void PrintDynamicArray(DynamicArray *da)
{
  unsigned int size = DynamicArraySize(da);

  printf("Dynamic array size: %d\n", size);
  printf("Dynamic array capacity: %d\n", DynamicArrayCapacity(da));

  printf("Elements:");
  unsigned int i;
  for ( i = 0; i < size; ++i) {
    printf(" %g", *ElementInDynamicArray(da, i));
  }
  printf("\n");
}


/*
  Function: ReadUnsignedInt
  Usage: u = ReadUnsignedInt(fp);
  -------------------------------
  Reads an unsigned integer from the file *fp. An error message is
  generated and the program exits if reading fails.
*/
static unsigned int ReadUnsignedInt(FILE* fp)
{
  unsigned int u;
  if ( fscanf(fp, "%ud", &u) < 1 ) {
    printf("Error reading input file\n");
    exit(2);
  }

  return u;
}


/*
  Function: ReadDouble
  Usage: d = ReadDouble(fp);
  --------------------------
  Reads a double from the file *fp. An error message is
  generated and the program exits if reading fails.
*/
double ReadDouble(FILE* fp)
{
  double d;
  if ( fscanf(fp, "%lf", &d) < 1 ) {
    printf("Error reading input file\n");
    exit(3);
  }

  return d;
}

/*
  Function: RandomUnsignedInteger
  Usage: u = RandomUnsignedInteger(n);
  ------------------------------------
  Returns a random unsigned integer between 0 and n, inclusive
*/
unsigned int RandomUnsignedInteger(unsigned int n)
{
  double rnd = drand48();
  double beyond_max = n + 1;
  return rnd * beyond_max;
}


/*
  Function: ReadDynamicArray
  Usage: da = ReadDynamicArray(fp);
  ---------------------------------
  Reads a dynamic array from the file *fp. The dynamic array is
  allocated using NewDynamicArray, and the elements read from the file
  *fp are added to the dynamic array using the InsertInDynamicArray
  function. After each insertion the dynamic array is printed.
*/
DynamicArray* ReadDynamicArray(FILE *fp)
{
  DynamicArray *da = NewDynamicArray();
  if ( da == NULL ) {
    printf("Cannot allocate memory for dynamic array\n");
    exit(4);
  }

  unsigned int i, size = ReadUnsignedInt(fp);
  void *p;
  for (i = 0; i < size; ++i) {
    p = InsertInDynamicArray(da, ReadDouble(fp));
    if ( p == NULL ) {
      printf("Cannot allocate memory for dynamic array\n");
      exit(4);
    }

    printf("\nvvvvvvv\n");
    PrintDynamicArray(da);
    printf("^^^^^^^\n\n");
  }

  return da;
}


/* Main program */
int main(int argc, char** argv)
{
  if ( argc < 2 ) {
    printf("\tUsage: %s <data file>\n", argv[0]);
    return 0;
  }

  FILE* in = fopen(argv[1], "r");
  if ( in == NULL ) {
    printf("Cannot open file %s for reading\n", argv[1]);
    return 1;
  }

  DynamicArray *da = ReadDynamicArray(in);


  printf("\n\n************\n\n");

  /* choose an element at random and delete it;
     do this until the dynamic array is empty */
  unsigned int size = DynamicArraySize(da);
  while ( size > 0 ) {
    unsigned int rnd = RandomUnsignedInteger(size);

    printf("\nvvvvvvv\n");
    printf("Deleting element at position: %d\n", rnd);
    if ( DeleteFromDynamicArray(da, rnd) == NULL ) {
      printf("Cannot allocate memory for dynamic array\n");
      exit(4);
    }
    PrintDynamicArray(da);
    printf("^^^^^^^\n\n");

    size = DynamicArraySize(da);
  }

  /* free the memory allocated for the dynamic array */
  FreeDynamicArray(da);

  return 0;
}
